function gpsResult = gpsdiff(gpsTime1,gpsTime2)
%GPSDIFF Difference time1-time2 in (S,NS) given two GPS times 
%
%SYNOPSIS
%   GPSRESULT = gpsdiff(GPSTIME1, GPSTIME2)
%
%INPUT
%   GPSTIME1    GPS time. Either integer seconds or struct with elements
%       sec, ns
%           sec: whole seconds
%           ns: residue in nano-seconds
%   GPSTIME2    GPS time to be subtracted from gpsTime1. Either integer
%       seconds or struct with elements sec, ns
%
%OUTPUT
%   GPSRESULT   Time difference gpsTime1 - gpsTime2. GPS time (struct
%       with elements sec, ns)
% 
% GPSTIME1 or GPSTIME2 may be 1-dimensional arrays. If both
% are arrays they must have same length. If input is an
% array, output is a struct array. 
%
% Sign convention if time < 0, .sec, .ns are both negative
%
%AUTHOR
%   Keith Thorne <keith.thorne@ligo.org>

% $Id: gpsdiff.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

diffSec = 0;
diffNsec = 0;
% IF # input arguments is wrong
%   CREATE error
% EXIT
error(nargchk(2,2,nargin),'struct')

% IF T1 is a GPS structure
%	GET sec, ns from structure
% ELSE
%   CONVERT time to GPS structure
%	GET sec, ns from structure
% END
% IF T2 is a GPS structure
%	GET sec, ns from structure
% ELSE
%   CONVERT time to GPS structure
%	GET sec, ns from structure
% END
if(isstruct(gpsTime1))
	t1Sec = [gpsTime1(:).sec];
	t1NSec = [gpsTime1(:).ns];
else
	t1 = time2gps(gpsTime1);
    t1Sec = [t1(:).sec];
	t1NSec = [t1(:).ns];
end
if(isstruct(gpsTime2))
	t2Sec = [gpsTime2(:).sec];
	t2NSec = [gpsTime2(:).ns];
else
	t2 = time2gps(gpsTime2);
    t2Sec = [t2(:).sec];
	t2NSec = [t2(:).ns];
end

% FIND differences in whole seconds (dS), nanoseconds (dNS)
% IF dS*dNS < 0
%     IF dS > 0 
%         dS  = dS-1;
%         dNS = dNS+nsecPerSec;
%     ELSE IF dS < 0
%         dS  = dS+1;
%         dNS = dNS-nsecPerSec;
%     END
%  END

% find differences
dS = t1Sec - t2Sec;
dNS = t1NSec - t2NSec;

% Put difference into canonical form
sgnS = sign(dS);
sgnNS = sign(dNS);
sgn = sgnS.*sgnNS;
sgn = sgn.*(sgn<0).*sgnS;

nsecPerSec = 1e9;

dS = dS + sgn;
dNS = dNS - sgn*nsecPerSec;

% try to match array orientation
if(length(dS) > 1)
    if(length(gpsTime1) > 1)
        [nRow,nCol] = size(gpsTime1);
    else
        [nRow,nCol] = size(gpsTime2);
    end        
    [testRow,testCol] = size(dS);
    if(testRow==nCol && testCol==nRow)
        dS = dS';
        dNS = dNS';
    end
end

gpsResult = struct('sec',num2cell(dS),'ns',num2cell(dNS));

return;
