function list = stdsegments(list);

%STDSEGMENTS Return list in standard form for further processing
%
% STDSEGMENTS verifies of converts the input list to the standard
% form for use by other functions in the data segments toolbox.
%
%SYNOPSIS
%   OUT = stdsegments(IN);
%
%INPUT
%   IN        input segment list
%
%OUTPUTS
%   OUT       output segment list
%
% The segments list should be provided in the form of an N-by-2 matrix,
% formed from adjacent column-wise vectors of segment start and stop
% times.
%
%   start1 stop1
%   start2 stop2
%   start3 stop3
%    ...    ...
%   startN stopN
%
% By definition, a segment is the half closed interval [start,stop).
% A segment is inclusive of the specified start time, but exclusive of
% the specified stop time.  The duration of a segment is defined as the
% difference between stop and start times.
%
% As an alternative, each segment may also be specified by its start time
% and duration.  However, the resulting segment list will be returned in
% terms of start and stop times.
%
% See also CULLSEGMENTS and MERGESEGMENTS.
%
%AUTHOR
%   Shourov K. Chatterji (No longer part of the LV Collaboration)

% $Id: stdsegments.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(1, 1, nargin));

% force column-wise lists
[rows, cols] = size(list);
if (rows == 0 | cols == 0),
  list = [];
  return;
elseif cols == 2,
elseif rows == 2,
  list = list';
else
  error('segment list is not N-by-2.');
end

% extract start and stop vectors
start = list(:,1);
stop = list(:,2);

% convert durations to stop times
if all(stop < start),
  stop = start + stop;
end

% create return segment list
list = [start stop];

% return to calling function
return;
