function [list, indices] = uniquesegments(list);

%UNIQUESEGMENTS Remove redundant segments from a segment list
%
% UNIQUESEGMENTS reads the requested segment list and removes all but
% one of redundant segments according to start time and stop time.
% The resulting unique segment list is returned as well as an index
% vector into the input segments list which selects only the unique
% segments.  UNIQUESEGMENTS also sorts the segment list based first
% on start time and next on stop time.
% 
%SYNOPSIS
%   [OUT, INDEX] = uniquesegments(IN);
%
%INPUT
%   IN        input segment list
%
%OUTPUT
%   OUT       output segment list
%   INDEX     index of unique segments in input list
%
% The index vector maps the input segment list to a set of unique output
% segments list in the following way.
%
% out = in(index, :);
%
% The segments list should be provided in the form of an N-by-2 matrix,
% formed from adjacent column-wise vectors of segment start and stop
% times.
%
%   start1 stop1
%   start2 stop2
%   start3 stop3
%    ...    ...
%   startN stopN
%
% By definition, a segment is the half closed interval [start,stop).
% A segment is inclusive of the specified start time, but exclusive of
% the specified stop time.  The duration of a segment is defined as the
% difference between stop and start times.
%
% As an alternative, each segment may also be specified by its start time
% and duration.  However, the resulting segment list will be returned in
% terms of start and stop times.
%
% See also NUMSEGMENTS, SORTSEGMENTS, and MERGESEGMENTS.
%
%AUTHOR
%   Shourov K. Chatterji (No longer part of the LV Collaboration)

% $Id: uniquesegments.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(1, 1, nargin));

% standardize segment list
list = stdsegments(list);

% handle empty lists
if isempty(list),
  indices = [];
  return;
end

% extract start and stop vectors
start = list(:,1);
stop = list(:,2);

% search list
list = [start stop];

% sort search list
[list, indices] = sortsegments(list, [1 2]);

% find unique segments
unique = find((diff(list(:, 1)) ~= 0) | (diff(list(:, 2)) ~= 0));
unique = [unique; length(start);];

% generate index of unique segments
indices = indices(unique);

% generate unique result list
list = list(unique, :);

% standardize segment list
list = stdsegments(list);

% return to calling function
return;
