function [sigmoidCenter, sigmoidWidth, sigmoidAsymmetry, ...
          sigmoidPedestal, sigmoidEntablature, efficiency, ...
          upperLimit, lowerLimit, chiSquare] = ...
         sigmoidfit(amplitude, nobserved, ninjected, doplot);

%SIGMOIDFIT Determine detection efficiencies and fit to sigmoid curve
%
%SYNOPSIS
%   [SIGMOIDCENTER, SIGMOIDWIDTH, SIGMOIDASYMMETERY, ...
%         SIGMOIDPEDESTAL, SIGMOIDENTABLATURE, EFFICIENCY, ...
%         UPPERLIMINT, LOWERLIMIT, CHISQUARE] = ...
%        sigmoidfit(AMPLITUDE, NOBSERVED, NINJECTED, DOPLOT);
%
%INPUTS
%   AMPLITUDE            vector of amplitudes
%   NOBSERVED            number of detected events at each amplitude
%   NINJECTED            number of injected events at each amplitude
%   DOPLOT               boolean flag to plot results
%
%OUTPUTS
%   SIGMOIDCENTER        amplitude at fifty percent efficiency
%   SIGMOIDWIDTH         approximate width of sigmoid curve in decades
%   SIGMOIDASYMMETRY     dimensionless asymmetry parameter
%   SIGMOIDPEDESTAL      minimum efficiency in small amplitude limit
%   SIGMOIDENTABLATURE   maximum efficiency in large amplitude limit
%   EFFICIENCY           vector of measured efficiencies
%   UPPERLIMIT           upper confidence limit on measured efficiency
%   LOWERLIMIT           lower confidence limit on measured efficiency
%   CHISQUARE            chi squared per degree of freedom for the fit
%
% If ninjected is scalar, SIGMOIDFIT assumes the same number of injections at
% all amplitudes.
%
% The reported upperLimit and lowerLimit correspond to sixty eight percent
% (roughly plus minus one sigma) confidence limits.
%
% If requested, the resulting sigmoid fit is plotted along with the measured
% efficiencies and confidence intervals.  By default, no plot is generated
% unless no output arguments are requested or a plot is requested.
%
% See also SIGMOID and SIGMOIDCHISQUARE.
%
%AUTHOR
% Lindy L. Blackburn <lindy.blackburn@ligo.org>
% Shourov K. Chatterji (No longer in the LV Collaboration)

% $Id: sigmoidfit.m 6645 2010-12-09 16:09:14Z diego.menendez@LIGO.ORG $

% parse command line arguments
error(nargchk(3, 4, nargin));

% do not plot by default
if nargin == 3,
  doplot = 0;
end

% plot results if not output arguments requested
if nargout == 0,
  doplot = 1,
end

% force row vectors
amplitude = amplitude(:);
nobserved = nobserved(:);
ninjected = ninjected(:);

% require at least three data points
if length(amplitude) < 5,
  error('sigmoidfit: at least five measurements are required');
end

% handle scalar ninjected
if length(ninjected) == 1,
  ninjected = ninjected * ones(size(nobserved));
end

% test for equal vector lengths
if length(nobserved) ~= length(amplitude),
  error('sigmoidfit: nobserved must be the same length as amplitude');
end
if length(ninjected) ~= length(amplitude),
  error('sigmoidfit: ninjected must be scalar or the same length as amplitude');
end

% sort by increasing amplitude
[amplitude, index] = sort(amplitude);
nobserved = nobserved(index);
ninjected = ninjected(index);

% determine one sigma confidence interval for fitting
[efficiency, confidenceInterval] = binofit(nobserved, ninjected, ...
                                           erfc(1 / (2 * sqrt(2))));

% estimate error in measured efficiency for fitting
efficiencyError = confidenceInterval(:, 2) - confidenceInterval(:, 1);

% determine plus minus one sigma confidence limits for output
[efficiency, confidenceInterval] = binofit(nobserved, ninjected, ...
                                           erfc(1 / sqrt(2)));
lowerLimit = confidenceInterval(:, 1);
upperLimit = confidenceInterval(:, 2);

% initial guess of fit parameters
[ignore, centerIndex] = min(abs(efficiency - 0.5));
sigmoidCenter = amplitude(centerIndex);
sigmoidWidth = 0.5;
sigmoidAsymmetry = -0.5;
sigmoidPedestal = min(efficiency);
sigmoidEntablature = max(efficiency);
fitParameters = [sigmoidCenter sigmoidWidth sigmoidAsymmetry ...
                 sigmoidPedestal sigmoidEntablature];

% perform least squares fit
options = optimset('MaxFunEvals', 1e4, 'MaxIter', 1e4);
fitParameters = fminsearch('sigmoidchisquare', fitParameters, options, ...
                           amplitude, efficiency, efficiencyError);

% extract fit parameters
sigmoidCenter = fitParameters(1);
sigmoidWidth = fitParameters(2);
sigmoidAsymmetry = fitParameters(3);
sigmoidPedestal = fitParameters(4);
sigmoidEntablature = fitParameters(5);

% find chi square of final fit
chiSquare = sigmoidchisquare(fitParameters, amplitude, efficiency, ...
                             efficiencyError);

% chi squared per degree of freedom
chiSquare = chiSquare / (length(amplitude) - 3);

% plot results
if doplot,
  clf;
  set(gca, 'FontSize', 14);
  sigmoidAmplitude = logspace(floor(log10(min(amplitude))), ...
                       ceil(log10(max(amplitude))), 1e3);
  sigmoidEfficiency = sigmoid(sigmoidAmplitude, sigmoidCenter, ...
                              sigmoidWidth, sigmoidAsymmetry, ...
                              sigmoidPedestal, sigmoidEntablature);
  handle = semilogx(sigmoidAmplitude, sigmoidEfficiency, 'b-');
  set(handle, 'LineWidth', 2);
  hold on;
  handle = semilogx(amplitude, efficiency, 'k.');
  set(handle, 'MarkerSize', 20);
  handle = semilogx([amplitude amplitude]', [lowerLimit upperLimit]', 'k-');
  set(handle, 'LineWidth', 2);
  axis([10^floor(log10(min(amplitude))) 10^ceil(log10(max(amplitude))) 0 1]);
  xlabel('Injected amplitude [strain Hz^{-1/2}]');
  ylabel('Detection efficiency');
  title('Detection efficiency vs. injected amplitude');
  grid on;
end
